//+------------------------------------------------------------------+
//|                                                   ma crosses.mq4 |
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1  Green
#property indicator_color2  Red

//
//
//
//
//

extern int    FastMaPeriod    = 8;
extern int    FastMaMethod    = MODE_EMA;
extern int    FastMaPrice     = PRICE_CLOSE;
extern int    SlowMaPeriod    = 33;
extern int    SlowMaMethod    = MODE_EMA;
extern int    SlowMaPrice     = PRICE_CLOSE;
extern bool   ShowTextOnly    = true;

extern string  MaMethods      = "Ma Types";           
extern string  __0            = "Sma";             
extern string  __1            = "Ema";
extern string  __2            = "Simple Smoothed Ma";
extern string  __3            = "Linear Weighted Ma";        
extern string  __4            = "Least Square Ma";

extern string note            = "turn on Alert = true; turn off = false";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";
extern int    arrowthickness  = 1;

extern bool   ShowText        = true;
extern string TextDir         = "Ma Text";
extern int    textXPosition   = 5;
extern int    textYPosition   = 30;
extern color  upTextcolor     = Blue;
extern color  dnTextcolor     = Red;
extern int    upFontsize      = 12;
extern int    dnFontsize      = 12;



//
//
//
//
//

double CrossUp[];
double CrossDn[];
double trend[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int init()
{
   IndicatorBuffers(3);   
   SetIndexBuffer(0,CrossUp);  
   SetIndexBuffer(1,CrossDn); 
   SetIndexBuffer(2,trend);
   
   if (ShowTextOnly)
   {
     SetIndexStyle(0,DRAW_NONE); 
     SetIndexStyle(1,DRAW_NONE); 
   }
   else
   {
     SetIndexStyle(0,DRAW_ARROW,0,arrowthickness); SetIndexArrow(0,233);
     SetIndexStyle(1,DRAW_ARROW,0,arrowthickness); SetIndexArrow(1,234);
   }       
   
   IndicatorShortName(" MA Cross ");
return(0);
}

//
//
//
//
//

int deinit()  
{   
   ObjectDelete(TextDir);
   
return(0);  
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int start() 
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
   
   
   for(i=limit; i>=0; i--)
   {
      double fastMa = iCustomMA(iMA(NULL,0,1,0,MODE_SMA,FastMaPrice,i),FastMaPeriod,FastMaMethod,i);               
      double slowMa = iCustomMA(iMA(NULL,0,1,0,MODE_SMA,SlowMaPrice,i),SlowMaPeriod,SlowMaMethod,i); 
        trend[i]   = trend[i+1];
           if (fastMa>slowMa) { trend[i] = 1; Commenter(1);  }
           if (fastMa<slowMa) { trend[i] =-1; Commenter(-1); }

           //
           //
           //
           //
           //
      
          CrossUp[i] = EMPTY_VALUE;
          CrossDn[i] = EMPTY_VALUE;
          if (trend[i]!= trend[i+1])
          if (trend[i] == 1)
                CrossUp[i] = Low[i]  - iATR(NULL,0,20,i)/2;
          else  CrossDn[i] = High[i] + iATR(NULL,0,20,i)/2;
         }
         
         //
         //
         //
         //
         //
      
         if (alertsOn)
         {
           if (alertsOnCurrent)
                int whichBar = 0;
           else     whichBar = 1;
           if (trend[whichBar] != trend[whichBar+1])
           if (trend[whichBar] == 1)
                 doAlert("Buy");
           else  doAlert("Sell");       
   }
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double workPrices[];
double workResult[];
int    r;

double iCustomMA(double price, int period, int method, int i)
{
   if (ArraySize(workPrices)!= Bars) ArrayResize(workPrices,Bars);
            r = Bars-i-1;
   workPrices[r] = price;

   //
   //
   //
   //
   //
   
      switch(method)
      {
         case  0: return(iSma(price,period,i));
         case  1: return(iEma(price,period,i));
         case  2: return(iSmma(price,period,i));
         case  3: return(iLwma(price,period,i));
         case  4: return(iLsma(price,period,i));
         
      }
   return(EMPTY_VALUE);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iSma(double price, double period, int i)
{
   double sum = 0;
   for(int k=0; k<period && (r-k)>=0; k++) sum += workPrices[r-k];  
   if (k!=0)
         return(sum/k);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iEma(double price, double period, int i)
{
   if (ArraySize(workResult)!= Bars) ArrayResize(workResult,Bars);
   double alpha = 2.0 / (1.0+period);
          workResult[r] = workResult[r-1]+alpha*(price-workResult[r-1]);
   return(workResult[r]);
}
//
//
//
//
//

double iSmma(double price, double period, int i)
{
   if (ArraySize(workResult)!= Bars) ArrayResize(workResult,Bars);
   if (i>=(Bars-period))
   {
      double sum = 0; 
         for(int k=0; k<period && (r-k)>=0; k++) sum += workPrices[r-k];  
         if (k!=0)
               workResult[i] = sum/k;
         else  workResult[i] = EMPTY_VALUE;
   }      
   else   workResult[r] = (workResult[r-1]*(period-1)+price)/period;
   return(workResult[r]);
}

//
//
//
//
//

double iLwma_prices[][3];
double iLwma(double price, double period, int i,int forValue=0)
{
   if (ArrayRange(iLwma_prices,0)!= Bars) ArrayResize(iLwma_prices,Bars);
   
   //
   //
   //
   //
   //
   
   iLwma_prices[r][forValue] = price;
      double sum  = 0;
      double sumw = 0;

      for(int k=0; k<period && (r-k)>=0; k++)
      {
         double weight = period-k;
                sumw  += weight;
                sum   += weight*iLwma_prices[r-k][forValue];  
      }             
   if (sumw!=0)
         return(sum/sumw);
   else  return(EMPTY_VALUE);
}

//
//
//
//
//

double iLsma(double price, double period, int i)
{
   return(3.0*iLwma(price,period,i)-2.0*iSma(price,period,i));
}

//
//
//
//
//

void Commenter(int Dir) 
{
    if (ShowText)
    {
      ObjectCreate(TextDir,OBJ_LABEL,0,0,0);
         ObjectSet(TextDir,OBJPROP_XDISTANCE,textXPosition);
         ObjectSet(TextDir,OBJPROP_YDISTANCE,textYPosition);
         if (Dir == 1) ObjectSetText(TextDir, "Long", upFontsize,"Arial Black",upTextcolor);
         if (Dir ==-1) ObjectSetText(TextDir, "Short",dnFontsize,"Arial Black",dnTextcolor);
    }
   
return; 
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ma cross ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," ma cross "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}
      